#!/bin/bash
#
# simple-sync-phone:bin/api/rsync-to-phone-config-api.sh
#
declare -gx PUB_KEY_FILE=

declare -g KEYFILES=

declare -gr WHIPTAIL_TITLE='Simple Sync phone'

declare -gr WHIPTAIL_CHECKLIST_TITLE='Choose public key to use'
declare -gi WHIPTAIL_CHECKLIST_HEIGHT=20
declare -gi WHIPTAIL_CHECKLIST_WIDTH=60
declare -gi WHIPTAIL_CHECKLIST_LISTHEIGHT=5

function requirements_config_init_pub_key_file {
  local -a tis=()
  local -a keyfile=
  while read -r keyfile ; do
    if [ -n "${keyfile}" ] ; then
      tis+=("${keyfile}" '' 'off') # tag item status
    fi
  done <<<"${KEYFILES}"

  local whiptail_result=
  local -i whiptail_rc=0
  whiptail_result="$(
    [ "${VERBOSE:-}" = "true" ] && set -x
    whiptail --separate-output \
    --title "${WHIPTAIL_TITLE}" \
    --checklist "${WHIPTAIL_CHECKLIST_TITLE}" \
    "${WHIPTAIL_CHECKLIST_HEIGHT}" "${WHIPTAIL_CHECKLIST_WIDTH}" "${WHIPTAIL_CHECKLIST_LISTHEIGHT}" \
    "${tis[@]}" 3>&1 1>&2 2>&3
  )"
  whiptail_rc=$?

  if [ "${VERBOSE:-}" = "true" ] ; then
    echodebug "whiptail_result='${whiptail_result}'"
    echodebug "whiptail_rc='${whiptail_rc}'"
  fi

  if [ "${whiptail_rc}" -ne 0 ] ; then
    return "${whiptail_rc}"
  fi

  PUB_KEY_FILE="${whiptail_result}"
}

function requirements_config {
  echoinfo 'Verify ssh key on local host'

  local ssh_keys=
  if ! ssh_keys="$( ssh-agent sh -c 'ssh-add -q; ssh-add -L -q' )"; then
    echowarn 'No ssh key found...'

    echoinfo 'Generating a new ssh key.'
    ssh-keygen -a 100 -t ed25519 -C "${USER}@${HOSTNAME}.$( date +'%Y-%m' )" -f "${HOME}/.ssh/id_ed25519" -N '' || return $?

    ssh_keys="$( ssh-agent sh -c 'ssh-add -q; ssh-add -L -q' )" || return $?
  fi

  local key_count=
  key_count="$( wc -l <<<"${ssh_keys}" )" || return $?
  echoinfo "Found '${key_count}' key(s)"

  local key=
  while read -r key ; do
    if [ -n "${key}" ] ; then
      echo '--------------------------'
      echo "${key}"
    fi
  done <<<"${ssh_keys}"
  echo '--------------------------'

  declare -g KEYFILES=
  KEYFILES="$( find ~/.ssh -type f -name 'id_*.pub' )" || return $?

  local keyfiles_count=
  keyfiles_count="$( wc -l <<<"${KEYFILES}" )" || return $?
  echoinfo "Found '${keyfiles_count}' public key file(s)"
}

function requirements_tools {
  local -a apps=()

  if ! which ssh >/dev/null ; then
    echowarn "ssh missing, need to install 'openssh-client'"
    apps+=('openssh-client')
  fi
  if ! which ip >/dev/null ; then
    echowarn "ip missing, need to install 'iproute2'"
    apps+=('iproute2')
  fi
  if ! which sed >/dev/null ; then
    echowarn "sed missing, need to install 'sed'"
    apps+=('iproute2')
  fi
  if ! which grep >/dev/null ; then
    echowarn "grep missing, need to install 'grep'"
    apps+=('grep')
  fi
  if ! which rsync >/dev/null ; then
    echowarn "rsync missing, need to install 'rsync'"
    apps+=('rsync')
  fi
  if ! which jq >/dev/null ; then
    echowarn "jq missing, need to install 'jq'"
    apps+=('jq')
  fi
  if ! which nmap >/dev/null ; then
    echowarn "nmap missing, need to install 'nmap'"
    apps+=('nmap')
  fi

  if [ "${#apps[@]}" -gt 0 ] ; then
    sudo apt install -y "${apps[@]}" || return $?
  else
    echookay 'Nothing to install (ok)'
  fi
}

